%% Define parameters
clear all;close all;
psf_sampling = 0.5e-6;%focal plane sampling in meters
lambda = 0.6328e-6;%wavelength in meters
N = 256;
aperture_diameter = 6e-03;%meters; 1 inch
focal_length = 1*aperture_diameter;%meters
RMS_SA = [0 0.4997 0.4997 0.5765 0.4074 0.4079];%RMS spherical aberration content in waves

%% Calculate pupil plane sampling
delta_fx = 1/(psf_sampling*N);
x_pupil = (-fix(N/2):fix((N-1)/2)) * delta_fx * lambda * focal_length;
[X_pupil,Y_pupil] = meshgrid(x_pupil);
R_pupil = sqrt(X_pupil.^2 + Y_pupil.^2);
X_norm = X_pupil/(aperture_diameter/2);
Y_norm = Y_pupil/(aperture_diameter/2);
R_norm = R_pupil/(aperture_diameter/2);%Pupil normalized to aperture diameter
assert(max(R_norm(:))>=sqrt(2),'Sampling is not sufficient to reconstruct the entire wavefront.');

%% Create wavefront
W = zeros(N,N,6);
W(:,:,2) = sqrt(2)*X_norm;%2
W(:,:,3) = sqrt(2)*RMS_SA(3) * Y_norm;%3
W(:,:,4) = sqrt(6)*RMS_SA(4) * (X_norm.^2 - Y_norm.^2);%4
W(:,:,5) =  sqrt(3)*RMS_SA(5) * (2*R_norm -1);%5
W(:,:,6) =  sqrt(6)*RMS_SA(6) * 2 * X_norm .* Y_norm;%6
for i = 1:N
    for j = 1:N
        for k = 2:6
            if R_norm(i,j)>1
            W(i,j,k)=0;
            end
        end
    end
end
W(R_norm>1) = 0;
E = exp(1i*2*pi*W);%Complex amplitude
E(R_norm>1) = 0;%Impose aperture size
figure;
for k = 1:6
    subplot(2,3,k);imagesc(angle(E(:,:,k))/(2*pi));
    axis([28 230 28 230]);
    axis off
    title(sprintf('%d',k));
end

%% Create point-spread function
for k = 1:6
    psf(:,:,k) = abs(fftshift(fft2(ifftshift(E(:,:,k))))).^2;
    psf(:,:,k) = psf(:,:,k)/sum(sum(psf(:,:,k)));
end
x_psf = (-fix(N/2):fix((N-1)/2)) * psf_sampling;
%for k = 1:6
%    figure;imagesc(x_psf*1e6,x_psf*1e6,psf(:,:,k));
%end

%% Create retinal image
input_image = imread('./Images/Flowers.png');
figure;
for k = 1:6
    retinal_image = imfilter(input_image, psf(:,:,k));
    subplot(2,3,k);imshow(retinal_image);
    title(sprintf('%d',k));
end
